#!/bin/sh
scriptdir=`dirname $0` ; cd ${scriptdir} ; scriptdir=`pwd` ; cd - > /dev/null

partedbin=`sudo which parted`

mountimg() {
  echo "Extracting ip/$1.img.gz"
  gzip -cd ${workdir}/iso/ip/$1.img.gz > ${workdir}/$1.img

  if [ $1 = gaius ]; then
    dd if=/dev/zero of=${workdir}/$1.img count=0 bs=1048576 seek=1024

#rm 1
#mkpart primary fat32 1048576B 1073741823B
#Ignore
#toggle 1 boot

    LANG=C $partedbin ${workdir}/$1.img <<EOF
unit b
print
resize 1 1048576B 1073741823B
resizepart 1 1073741823B
quit
EOF

    LOOPDEV=`sudo losetup --find --show -o 1048576 --sizelimit 1072693248 ${workdir}/$1.img`

    lbl=`sudo fatlabel $LOOPDEV`
    uid=`sudo blkid /dev/loop0 -o export | grep -E '^UUID=' | cut -d '=' -f 2 | tr -d -`

    sudo mkfs.vfat -F 32 -h 2048 -i $uid -n $lbl $LOOPDEV
    sudo fatlabel $LOOPDEV
    sudo losetup -d $LOOPDEV
  fi

  ofs=0
  sze=0
  LANG=C $partedbin ${workdir}/$1.img unit b print
  LANG=C $partedbin ${workdir}/$1.img unit b print 2>/dev/null | grep -m1 fat | while read n b e s t f z ; do
    ofs=`echo $b | tr -d B`
    sze=`echo $s | tr -d B`

    #echo mount -o loop,ro,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount
    sudo mount -o loop,offset=$ofs,sizelimit=$sze ${workdir}/$1.img ${workdir}/mount
  done
}

cpdir(){
  if [ -e $1 ]; then
    sudo cp -ap --no-preserve=ownership $1 $2
  fi
}

cpfile(){
  if [ -e $1 ]; then
    sudo cp -p --no-preserve=ownership $1 $2
  fi
}

isofile=HPIP.iso
for f in `ls -tr HPIP*.iso` ; do
  isofile=$f
done

if [ ! -e ${isofile} ]; then
  isofile=HPIP164B.2017_0201.3.iso
  wget -N https://downloads.hpe.com/pub/softlib2/software1/cd/p1450150448/v125952/HPIP164B.2017_0201.3.iso
fi

[ ! -e ${isofile} ] && exit 1

workdir=${isofile/.iso/}

if [ ! -e ${workdir} ]; then
  bash hpip-extract.sh
fi

partedbin=`sudo which parted`

echo ""
echo Using ${workdir}


liveiso=Debian.iso
for f in `ls -tr *bian*.iso *buntu*.iso *live*.iso *magic*.iso *repair*.iso *rescue*.iso *7pe*.iso *win*.iso *7PE*.iso *Win*.iso *ccmulti*.iso 2>/dev/null` ; do
  liveiso=$f
done

if [ ! -e ${liveiso} ]; then
  liveiso=gparted-live-0.29.0-1-amd64.iso
  wget -N https://downloads.sourceforge.net/project/gparted/gparted-live-stable/0.29.0-1/gparted-live-0.29.0-1-amd64.iso
fi

[ ! -e ${liveiso} ] && exit 1

livedir=${liveiso/.iso/}
liveName=`echo ${livedir} | sed s/'[-_.][0-9_.-]*[-_]'/'-'/g | sed s/'[-_][0-9.-]*$'/''/g | sed s/'-64bit'/''/g | sed s/'-amd64+mac'/''/g | sed s/'-amd64'/''/g | sed s/'-x64'/''/g | tr - ' ' | tr _ ' '`

echo Using ${liveName} in ${livedir}

sudo mount -o loop,ro ${liveiso} ${workdir}/mount

mkdir -p ${workdir}/liveiso
cp -apP --no-preserve=ownership ${workdir}/mount/. ${workdir}/liveiso/
sudo chmod -R ugo+rw ${workdir}/liveiso/
sudo umount ${workdir}/mount

cd ${workdir}/liveiso/

liveMode=unknown

if [ -e casper/filesystem.squashfs ]; then
  liveMode=casper

elif [ -e live/filesystem.squashfs ]; then
  liveMode=live

elif [ -e pmagic/pmodules ]; then
  liveMode=pmagic

elif [ -e sources/boot.wim ]; then
  liveMode=winpe

elif [ -e sources/BOOT.WIM ]; then
  liveMode=winpe

else
  echo "Unknown live ISO format"
  exit 1
fi

if [ $liveMode = winpe ]; then
  liveInit=boot.sdi

  if [ -e boot/boot.sdi ]; then
    liveInit=boot/boot.sdi
  elif [ -e BOOT/BOOT.SDI ]; then
    liveInit=BOOT/BOOT.SDI
  fi

  liveKern=ntldr

  if [ -e bootmgr ]; then
    liveKern=bootmgr
  elif [ -e BOOTMGR ]; then
    liveKern=BOOTMGR
  fi

else
  liveInit=initrd

  if [ -e ${liveMode}/initrd.lz ]; then
    liveInit=initrd.lz
  elif [ -e ${liveMode}/initrd.gz ]; then
    liveInit=initrd.gz
  elif [ -e ${liveMode}/initrd.img ]; then
    liveInit=initrd.img
  elif [ -e ${liveMode}/minirt.gz ]; then
    liveInit=minirt.gz
  fi

  if [ ! -e ${liveMode}/${liveInit} ]; then
    echo "Unknown live ISO format, initrd not found"
    exit 1
  fi

  liveKern=vmlinux

  if [ -e ${liveMode}/vmlinuz ]; then
    liveKern=vmlinuz
  elif [ -e ${liveMode}/vmlinuz.efi ]; then
    liveKern=vmlinuz.efi
  elif [ -e ${liveMode}/bzImage64 ]; then
    liveKern=bzImage64
  elif [ -e ${liveMode}/bzImage ]; then
    liveKern=bzImage
  elif [ -e ${liveMode}/linux ]; then
    liveKern=linux
  fi

  if [ ! -e ${liveMode}/${liveKern} ]; then
    echo "Unknown live ISO format, kernel not found"
    exit 1
  fi
fi

cd ${scriptdir}

mountimg gaius

sudo cp -ap --no-preserve=ownership ${workdir}/gaius/. ${workdir}/mount/

cd ${workdir}/mount

#sudo rm boot/initrd.img
#sudo rm boot/vmlinuz
#sudo rm -rf hotfixes/
#sudo rm system/squashfs.img

sudo rm sources/boot.wim

if [ ! -e ${scriptdir}/grub.cfg-${liveMode} ]; then
  sudo cp -p --no-preserve=ownership ${scriptdir}/grub.cfg-live ./boot/grub/grub.cfg
  sudo sed -i "s|/live/vmlinuz|/${liveMode}/vmlinuz|g" ./boot/grub/grub.cfg
  sudo sed -i "s|/live/initrd|/${liveMode}/initrd|g" ./boot/grub/grub.cfg
else
  sudo cp -p --no-preserve=ownership ${scriptdir}/grub.cfg-${liveMode} ./boot/grub/grub.cfg
fi

livesrc=${scriptdir}/${workdir}/liveiso

if [ $liveMode = winpe ]; then
  df -h .

  sudo cp -p --no-preserve=ownership ${livesrc}/sources/boot.wim ./sources/

else
  sudo rm -f bootmgr
  sudo rm -f bootmgr.efi

  sudo rm -f boot/BCD
  sudo rm -f boot/bootfix.bin
  sudo rm -f boot/boot.sdi

  sudo rm -rf EFI/Microsoft/

  sudo rm -rf sources/
  sudo rm -rf win/
  sudo rm -rf winpe/

  df -h .

  sudo sed -i 's|'${liveMode}/initrd.lz'|'${liveMode}/${liveInit}'|g' ./boot/grub/grub.cfg


  cd ${livesrc}

  lxcmdline="boot=${liveMode}"

  liveLdr=syslinux
  liveCfg=isolinux.cfg

  if [ -e boot/syslinux/syslinux.cfg ]; then
    liveCfg=boot/syslinux/syslinux.cfg
  fi
  if [ -e boot/isolinux/isolinux.cfg ]; then
    liveCfg=boot/isolinux/isolinux.cfg
  fi
  if [ -e syslinux/isolinux.cfg ]; then
    liveCfg=syslinux/isolinux.cfg
  fi
  if [ -e isolinux/isolinux.cfg ]; then
    liveCfg=isolinux/isolinux.cfg
  fi
  if [ -e isolinux/txt.cfg ]; then
    liveCfg=isolinux/txt.cfg
  fi

  if [ -e ${liveCfg} ]; then
    lxcmdtemp=`cat ${liveCfg} | grep -m1 -E 'append.*initrd='/${liveMode}/${liveInit}`
    if echo X${lxcmdtemp} | grep -q -m1 -E 'append.*initrd='/${liveMode}/${liveInit} ; then
      lxcmdline='linux	'/${liveMode}/${liveKern}'  '`echo $lxcmdtemp | sed s/'.*append'/''/g | sed 's|initrd='/${liveMode}/${liveInit}'||g'`

    else
      lxcmdtemp=`cat ${liveCfg} | grep -m1 -A1 -E 'INITRD.*'/${liveMode}/${liveInit} | grep APPEND`
      if echo X${lxcmdtemp} | grep -q APPEND ; then
        lxcmdline='linux	'/${liveMode}/${liveKern}'  '`echo $lxcmdtemp | sed s/'.*APPEND'/''/g`
      fi
    fi
  fi

  tempLdr=none
  tempCfg=grub.cfg

  if [ -e boot/grub/loopback.cfg ]; then
    tempLdr=grub
    tempCfg=boot/grub/loopback.cfg
  fi
  if [ -e boot/grub/grub.cfg     ]; then
    tempLdr=grub
    tempCfg=boot/grub/grub.cfg
  fi

  if [ $tempLdr = grub -a -e ${tempCfg} ]; then
    lxcmdtemp=`cat ${tempCfg} | sed s/'^[ \t]*linux'/'linux'/g | grep -m1 -E 'linux.*'/${liveMode}/${liveKern}`
    if echo X${lxcmdtemp} | grep -q -m1 -E 'linux.*'/${liveMode}/${liveKern} ; then
      liveLdr=${tempLdr}
      liveCfg=${tempCfg}
      lxcmdline=$lxcmdtemp
    fi
  fi


  cd ${scriptdir}/${workdir}/mount

  cpdir ${livesrc}/.disk ./
  cpdir ${livesrc}/dists ./
  cpdir ${livesrc}/install ./
  cpdir ${livesrc}/pool ./
  cpdir ${livesrc}/preseed ./

  sudo cp -ap --no-preserve=ownership ${livesrc}/${liveMode} ./

  echo $lxcmdline
  if echo $lxcmdline | grep -q ${liveKern} ; then
    lxcmdtoram=`echo ${lxcmdline} | sed s/'boot='${liveMode}' '/'boot='${liveMode}' toram '/g`

    echo $lxcmdtoram
    echo got cmdline from ${liveCfg}

    #sed -i 's|linux.*'/${liveMode}/vmlinuz'.*|'${lxcmdline}'|g' ./boot/grub/grub.cfg
    sudo sed -i "s|linux.*/${liveMode}/vmlinuz.*toram.*|xXlinux-cmdline-toram-hereXx|g" ./boot/grub/grub.cfg
    sudo sed -i "s|linux.*/${liveMode}/vmlinuz.*|${lxcmdline}|g" ./boot/grub/grub.cfg
    sudo sed -i "s|xXlinux-cmdline-toram-hereXx|${lxcmdtoram}|g" ./boot/grub/grub.cfg
  else
    if [ ! -e ./${liveMode}/vmlinuz ]; then
      #sudo cp -p ./${liveMode}/${liveKern} ./${liveMode}/vmlinuz
      sudo sed -i "s|/${liveMode}/vmlinuz|/${liveMode}/${liveKern}|g" ./boot/grub/grub.cfg
    fi
  fi

  sudo sed -i "s|Debian-XFCE|${liveName}|g" ./boot/grub/grub.cfg
  sudo sed -i "s|Xubuntu|${liveName}|g" ./boot/grub/grub.cfg

  cpfile ${livesrc}/md5sum.txt ./
  cpfile ${livesrc}/README.diskdefines ./
  cpfile ${livesrc}/ubuntu ./

  cpfile ${livesrc}/*-Live-* ./
  cpfile ${livesrc}/GPL ./
fi

df -h .

cd ${scriptdir}

echo ""
echo "You can make your own changes in ${workdir}/mount now, if required."
echo "Press Enter when done."
read f;

sudo umount ${workdir}/mount

mkdir -p ${workdir}/combinediso
cp -apP ${workdir}/iso/. ${workdir}/combinediso/

echo "Creating ip/gaius.img.gz"
cat ${workdir}/gaius.img | gzip -c > ${workdir}/combinediso/ip/gaius.img.gz

cd ${workdir}/combinediso/ip/

sudo rm md5s.txt
sudo rm sha1sums.txt
md5sum *.img.gz > md5s.txt
shasum *.img.gz > sha1sums.txt

cd ${scriptdir}

#geteltorito ${isofile} > ${workdir}/eltorito.bin
#cp -p ${workdir}/eltorito.bin ${workdir}/combinediso/boot/grub/stage2_eltorito

IMAGE=`basename ${isofile} | cut -d _ -f 1 | cut -d . -f 1`-${liveMode}.iso
IMAGE_NAME=`basename ${isofile} | cut -d _ -f 1 | cut -d . -f 1`

cd ${workdir}/combinediso/

if false; then
  echo mkisofs -U -V "$IMAGE_NAME" \
            -J -joliet-long -r -v -T -b boot/grub/stage2_eltorito -c boot.catalog -no-emul-boot -boot-load-size 4 -boot-info-table -no-emul-boot \
            -o ../${IMAGE} .

  mkisofs -U -V "$IMAGE_NAME" \
            -J -joliet-long -r -v -T \
            -b boot/grub/stage2_eltorito -c boot.catalog -no-emul-boot -boot-load-size 4 -boot-info-table -no-emul-boot \
            -o ../${IMAGE} .
else

  LANG=C grub-mkrescue -o ../${IMAGE} . -- -volid "$IMAGE_NAME"
fi
